package de.hdm_stuttgart.sw1.klausur.string;

import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Hilfsklasse zur Bereitstellung String-bezogener Methoden.
 *
 */
public class StringHelper {

	/**
	 * Bestimmung der Geamtlänge einer Folge von Worten. Beispiel:
	 * 
	 * <code>{"Dies", " ist ", " kurz.", null} ist ein Array aus drei Strings und einem null Wert mit in Summe 
	 *   4 +      5 +      6 +     0 = 15 Zeichen.</code>
	 * 
	 * @param words
	 *            Ein Array aus Strings. Kann null sein, null Werte enthalten
	 *            oder die Länge 0 haben.
	 * 
	 * @return Die Gesamtlänge aller Worte. Enthaltene null Werte werden als
	 *         Länge 0 gewertet. Falls null == words, ist der Rückgabewert 0.
	 */
	public static int summierteWortLaenge(final String[] words) {
		int gesamtLaenge = 0;
		if (null != words) {
			for (final String w : words) {
				if (null != w) {
					gesamtLaenge += w.length();
				}
			}
		}
		return gesamtLaenge;
	}

	/**
	 * Bestimmung der Anzahl von Ziffern in einer gegebenen Zeichenkette.
	 * 
	 * Beispiele:
	 * 
	 * "Anton" enthält 0 Ziffern. "4Laufen 1-2-3" enthält vier Ziffern.
	 * 
	 * @param s
	 *            Die zu untersuchende Zeichenkette.
	 * @return Die Anzahl der in s enthaltenen Ziffern. Für null == s soll der
	 *         Wert 0 zurückgegeben werden.
	 */
	public static int anzahlZiffern(final String s) {
		int numAsciiCharacters = 0;
		if (null != s) {
			for (int i = 0; i < s.length(); i++) {
				if (Character.isDigit(s.charAt(i))) {
					numAsciiCharacters++;
				}
			}
		}
		return numAsciiCharacters;
	}

	/**
	 * Korrespondierende Methode zu {@link #anzahlZiffern(String)}: Die
	 * Ziffernsuche wird auf alle in einem Array enthaltenen Strings ausgedehnt.
	 * 
	 * @param strings
	 *            Liste übergebener Strings.
	 * 
	 * @return Die Anzahl aller Ziffern aus allen im Array übergebenen Strings.
	 *         Enthaltene null Werte werden ignoriert.
	 *         Ein null Array liefert 0 Ziffern. Ein leeres Array liefert den Wert 0
	 */
	public static int anzahlZiffern(final String strings[]) {
		int numAsciiCharacters = 0;
		if (null != strings) {
			for (final String s : strings) {
				numAsciiCharacters += anzahlZiffern(s);
			}
		}
		return numAsciiCharacters;
	}

	/**
	 * Entferne alle doppelt auftretenden Zeichenketten und null Werte aus einem
	 * String Array mit Unterscheidung von Groß- und Kleinschreibung und ordne
	 * das Ergebnis alphabetisch gemäß {@link String#compareTo(String)}.
	 * Beispiel:
	 * 
	 * {"Haus", "Wiese", null, "laufen", null, "Wiese", "Kiste"}
	 * 
	 * Das erwartete Ergebnis ist:
	 * 
	 * {"Haus", "Kiste", "Wiese", "laufen"}
	 * 
	 * Die Reihenfolge ergibt sich aus der alphabetischen Sortierung gemäß
	 * {@link String#compareTo(String)} unter Beachtung von Groß- und
	 * Kleinschreibung.
	 * 
	 * @param inputStrings
	 *            Eine Array aus Strings mit möglichen Duplikaten und null
	 *            Werten. Ein Argument inputStrings == null wird wie ein Array
	 *            der Länge 0, also als {}, gewertet.
	 * 
	 * @return Die enthaltenen Strings ohne Duplikate und null Werte in
	 *         alphabetisch aufsteigender Reihenfolge. Der Rückgabewert ist
	 *         garantiert von null verschieden.
	 */
	public static String[] filterUndSortiereDuplikate(final String[] inputStrings) {

		if (null == inputStrings) {
			return new String[0];
		} else {
			final SortedSet<String> uniqueNonNullValues = new TreeSet<String>();
			for (final String s : inputStrings) {
				if (null != s) {
					uniqueNonNullValues.add(s);
				}
			}
			return uniqueNonNullValues.toArray(new String[0]);
		}
	}
}