package de.hdm_stuttgart.mi.sd1.aufgabe1;

/**
 * <p>Ein Fitnesstudio bietet folgende Tarife:</p>
 *
 * <ul>
 *   <li>Jährliche Einmalzahlung, keine Kosten pro Besuch, siehe {@link Tarif#JAHRESFLATRATE}.</li>
 *   <li>Jährliche Vorauszahlung und Kosten pro Besuch, siehe {@link Tarif#GEMISCHT}.</li>
 *   <li>Nur Kosten pro Besuch, keine jährlichen Kosten, siehe {@link Tarif#PRO_BESUCH}.</li>
 * </ul>
 *
 * <p>Der Rechner ermittelt sowohl die Kosten als auch die günstigste Tarifart auf Basis der Anzahl von
 *    Besuchen pro Jahr.</p>
 */
abstract public class FitnessTarifRechner {

  /**
   * Tarifarten zur Nutzung des Studios. Für jede Tarifart sind die Jahreskosten {@link #jahresKosten} und die
   * Kosten pro Besuch {@link #kostenProBesuch} verfügbar.
   */
  public enum Tarif {
    /**
     * Keine Jahresbeitrag, nur {@link #kostenProBesuch}
     */
    PRO_BESUCH(0, 12),

    /**
     * Jahresbeitrag  {@link #jahresKosten} und {@link #kostenProBesuch}.
     */
    GEMISCHT(150, 5),

    /**
     * Keine Kosten pro Besuch, Jahresbeitrag {@link #jahresKosten}.
     */
    JAHRESFLATRATE(800, 0);

    Tarif(final int jahresKosten, final int kostenProBesuch) {
      this.jahresKosten = jahresKosten;
      this.kostenProBesuch = kostenProBesuch;
    }

    /**
     * Gesamtkosten auf Basis einer jährlichen Vorauszahlung und Kosten pro Besuch.
     *
     * @param anzahlBesuche Negative Werte werden als «0 Besuche» gewertet.
     * @return Die jährlichen Gesamtkosten.
     */
    public int getKosten(final int anzahlBesuche) {
      if (anzahlBesuche <= 0) {
        return jahresKosten;
      } else {
        return jahresKosten + anzahlBesuche * kostenProBesuch;
      }
    }

    /**
     * Jährlich anfallende Kosten in der jeweiligen Tarifart.
     */
    public final int jahresKosten;

    /**
     * Pro Besuch des Studios anfallende Kosten.
     */
    public final int kostenProBesuch;

  }

  /**
   * Jährliche Gesamtkosten Kosten bei Nutzung des {@link Tarif#GEMISCHT} Tarifs.
   *
   * @param anzahlBesucheProJahr Anzahl der jährlichen Besuche im Studio. Negative Werte werden als
   *                             «0 Besuche» gewertet.
   * @return Die Gesamtkosten aller Besuche im Jahr.
   */
  static public int getKosten_GEMISCHT_Tarif(final int anzahlBesucheProJahr) {
    return Tarif.GEMISCHT.getKosten(anzahlBesucheProJahr);
  }

  /**
   * Jährliche Gesamtkosten Kosten ohne Buchung eines Tarifs.
   *
   * @param anzahlBesucheProJahr Anzahl der jährlichen Besuche im Studio. Negative Werte werden als
   *                             0 Besuche gewertet.
   * @return Die Gesamtkosten aller Besuche im Jahr.
   */
  static public int getKostenOhneTarif(final short anzahlBesucheProJahr) {
    return Tarif.PRO_BESUCH.getKosten(anzahlBesucheProJahr);
  }
  /**
   * <p>Optimaler Tariftyp basierend auf der Anzahl von Besuchsterminen pro Jahr.</p>
   *
   * <p>Beispiel: Ein Nutzer besucht das Studio an <span style="color:red">200</span> Tagen im Jahr. Dies ergibt folgende
   * Gesamtkosten in den drei Tariftypen:</p>
   *
   * <table class="goikTableDefaults">
   *   <tr>
   *      <th>Tariftyp</th>
   *      <th>Jährliche Kosten</th>
   *   </tr>
   *   <tr>
   *    <td>{@link Tarif#JAHRESFLATRATE}</td>
   *    <td>800.- € für Jahresflatrate</td>
   *   </tr>
   *
   *   <tr>
   *    <td>{@link Tarif#GEMISCHT} (Jahreskosten 150.- € und 5.- € pro Besuch)</td>
   *    <td>150.- € + <span style="color:red">200</span> * 5.- €</td>
   *   </tr>
   *
   *   <tr>
   *    <td>{@link Tarif#PRO_BESUCH} (12.-€ pro Besuch)</td>
   *    <td><span style="color:red">200</span> * 12.- €</td>
   *   </tr>
   *
   * </table>
   *
   * @param anzahlBesucheProJahr Anzahl der jährlichen Besuche im Studio.
   *
   * @return <p>Der günstigste Tariftyp bei vorgegebener Anzahl an Besuchen pro Jahr. Bei identischen Kosten
   * für verschiedene Tariftypen gilt folgende Prioritätenfolge:</p>
   * <ol>
   *   <li>{@link Tarif#PRO_BESUCH}</li>
   *   <li>{@link Tarif#GEMISCHT}</li>
   *   <li>{@link Tarif#JAHRESFLATRATE}</li>
   * </ol>
   *
   */

  static public Tarif getOptimalenTariftyp(final short anzahlBesucheProJahr) {

    final int
        flatrate = Tarif.JAHRESFLATRATE.getKosten(anzahlBesucheProJahr),
        gemischt = Tarif.GEMISCHT.getKosten(anzahlBesucheProJahr),
        proBesuch = Tarif.PRO_BESUCH.getKosten(anzahlBesucheProJahr);


    if (proBesuch <= gemischt && proBesuch <= flatrate) {
      return Tarif.PRO_BESUCH;
    } else  { // proBesuch ist nicht der günstigste Tarif, dann muss es einer der zwei übrigen Tarife sein:
      if (gemischt <= flatrate) {
        return Tarif.GEMISCHT;
      } else {
        return Tarif.JAHRESFLATRATE;
      }
    }
  }
}