package de.hdm_stuttgart.mi.sd1.aufgabe2;

import java.text.DecimalFormat;

/**
 * <p>Längen- oder Breitenangaben in Grad, Winkelminuten und Winkelsekunden. Dabei gilt:</p>
 *
 * <ul>
 *   <li>360° Grad entsprechen dem Vollkreis.</li>
 *   <li>60 Winkelminuten ergeben 1 Grad: 60´ == 1°.</li>
 *   <li>60 Winkelsekunden ergeben eine Winkelminute: 60´´ = 1´.</li>
 * </ul>
 *
 * <p>Beispiel: Die geographische Breitenangabe von Stuttgart lautet <b style="color: red;">48°</b>
 * <b style="color: green;">46′</b> <b style="color: blue;">56″</b> (Nord, wird in dieser Aufgabe ignoriert). Diese
 * Darstellung als Grad, Bogenminuten und Bogensekunden kann in die dezimale Grad-Darstellung umgerechnet werden:</p>
 *
 * <p><b style="color: red;">48</b> + <b style="color: green;">46</b>/60 + <b style="color: blue;">56</b> / 3600
 * == 48.78222222222...</p>
 *
 * <p>Die Klasse {@link Geo (double)} kann zur Umwandlung von Grad/Minuten/Sekundenangaben in
 * Dezimalkoordinaten und umgekehrt verwendet werden:</p>
 *
 */
public class Geo implements Comparable<Geo> {

  private static DecimalFormat df = new DecimalFormat();
  static {
    df.setMaximumFractionDigits(6);
    df.setMinimumFractionDigits(6);
  }

  static boolean nutzeDezimal = false;

  /**
   * Die {@link #toString()} erzeugt im Standard einen String der Form »48° 23' 11''«. Alternativ
   * kann auch die dezimale Graddarstellung der Art »48.78222222...« als Default festgelegt werden.
   * @param nutzeDezimalDefault Für true erzeugt {@link #toString()} die Darstellung  »48°,23',11''«,
   *                          ansonsten »48.78222222...«.
   */
  public static void nutzeDezimalStandard(final boolean nutzeDezimalDefault) {
    nutzeDezimal = nutzeDezimalDefault;
  }

  /**
   * Winkelangabe in Grad: Ein Vollkreis entspricht 360°.
   */
  public final int grad;
  /**
   * Winkelminuten: 60′ (sprich 60 Winkelminuten) entsprechen 1°.
   */
  public final int minuten;
  /**
   * Winkelsekunden: 60″ (sprich 60 Winkelsekunden) entsprechen 1′.
   */
  public final int sekunden;


  /**
   * Erzeuge Längen- oder Breitengradangabe.
   *
   * @param grad Winkelanteil in Grad
   * @param minuten Winkelanteil in Minuten
   * @param sekunden Winkelanteil in Sekunden
   */
  public Geo(final int grad, final int minuten, final int sekunden) {
    this.grad = grad;
    this.minuten = minuten;
    this.sekunden = sekunden;
  }

  /**
   * Längen- oder Breitenangabe aus Dezimaldarstellung.
   *
   * @param gradDezimal Dezimale Gradangabe, z.B. 43.221234
   */
  public Geo(final double gradDezimal) {

    final double grad = Math.floor(gradDezimal);

    final double minutenSekunden = 60 * (gradDezimal - grad);

    final double minutes = Math.floor(minutenSekunden);

    final double sekunden = 60 * (minutenSekunden - minutes);

    this.grad = (int) grad;
    this.minuten = (int)minutes;
    this.sekunden = (int) Math.round(sekunden);
  }

  public double getDezimal() {
    return grad + (minuten + sekunden / 60.) / 60;
  }

  @Override
  public String toString() {
    if (nutzeDezimal) {
      return df.format(getDezimal());
    } else {
      return grad + "° " + minuten + "´ " + sekunden + "´´";
    }
  }

  @Override
  public int compareTo(final Geo o) {

    if (grad == o.grad) {
      if (minuten == o.minuten) {
        return sekunden - o.sekunden;
      } else {
        return minuten - o.minuten;
      }
    } else {
      return grad - o.grad;
    }
  }

  @Override
  public boolean equals(final Object other) {
    if (other instanceof Geo) {
      final Geo g = (Geo) other;

      return grad == g.grad && minuten == g.minuten && sekunden == g.sekunden;
    } else {
      return false;
    }
  }

  @Override
  public int hashCode() {
    return 3600 * grad + 60 * minuten + sekunden;
  }
}