package de.hdm_stuttgart.mi.sda1.reewriteprepared;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.junit.Rule;
import org.junit.FixMethodOrder;
import org.junit.runners.MethodSorters;

import de.hdm_stuttgart.mi.sda1.reewriteprepared.rdbms.CustomerInsert;

/**
 * Creating a database connection, an initial schema and one record in {{@link #initDatabase()}. Subsequent tests
 * try to add more records and perform related checks.
 */

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
@SuppressWarnings({"UnusedDeclaration", "javadoc", "resource"})
public class CustomerInsertTest {

   @Rule // Kill test if tests last longer than 5000 milliseconds unless in debugging mode
   public final TestRule timeout = new DisableOnDebug(Timeout.millis(5000));

   Connection connection = null;
   CustomerInsert customerInsert = null;
   
   // Create schema from scratch and insert a customer record "jim" prior to execution of test methods
   @Before
   public void initDatabase() throws SQLException {

      connection = DriverManager.getConnection (
            "jdbc:mysql://localhost:3306/hdm", "hdmuser", "XYZ");

      customerInsert = new CustomerInsert(connection); // Will set auto commit to true

      
      final Statement statement = connection.createStatement(); 

      statement.executeUpdate("DROP TABLE IF EXISTS Customers");

      statement.executeUpdate(
            "CREATE TABLE Customers (\n"
            + "uidNumber INTEGER NOT NULL AUTO_INCREMENT PRIMARY KEY,\n"
            + "uid CHAR(20) NOT NULL UNIQUE,\n"
            + "cname VARCHAR(255) NOT NULL\n" +
            ");");

      statement.executeUpdate("INSERT INTO Customers (uid, cname) VALUES ('jim', 'Jim Bone');");
      connection.commit();
      statement.close();
   }
   
   /**
    * @throws SQLException Failure should not occur during test phase
    */
   @Test
   public void testInsertDuplicate() throws SQLException {
      // "jim" was inserted by initDatabase already, so the subsequent call is bound to fail
      Assert.assertFalse(customerInsert.conditionallyInserCustomer("jim", "Jim Alternate"));
   }
   
   /**
    * @throws SQLException Failure should not occur during test phase
    */
   @Test
   public void testInsertNew() throws SQLException {
      
      // Insert second customer on top "jim" from initialization phase
      //
      Assert.assertTrue(customerInsert.conditionallyInserCustomer("eve", "Eve Gardener"));

      // Search for all customers We expect two customers "eve" followed by "jim" 
      // to be present in our database.
      final ResultSet customers = connection.createStatement().executeQuery(
             "SELECT uid FROM Customers ORDER BY uid");
      
      // Check for "eve"
      Assert.assertTrue("No customer record found: Should be two", customers.next());
      Assert.assertEquals("eve", customers.getString("uid"));
      
      // Check for "jim"
      Assert.assertTrue("Just one customer record found: Should be two", customers.next());
      Assert.assertEquals("jim", customers.getString("uid"));
      
      // No further record should be present
      Assert.assertFalse("More than two customer records found", customers.next());
   }
}
