package de.hdm_stuttgart.mi.marking.implement;


/**
 * Creating mark overviews.
 */
public class MarkFrequencies {

  /**
   * <p>Mark frequencies:</p>
   *
   * <ul>
   *   <li>The first element holds the lowest mark's frequency.</li>
   *   <li>The last element holds the highest mark's frequency.</li>
   * </ul>
   *
   * <p>Consider an example:</p>
   *
   * <pre>{-3, 4 , 2, -2, 4, -3, 2, 4}</pre>
   *
   * <p>The resulting frequencies are:</p>
   *
   * <pre>
   * Value | frequency
   * ------+---------
   *    -3 | 2
   *    -2 | 1
   *    -1 | 0
   *     0 | 0
   *     1 | 0
   *     2 | 2
   *     3 | 0
   *     4 | 3
   * </pre>
   *
   * <p>The lowest mark -3 will be stored in {@link #lowestMark}. The value of {@link #frequencyByMark}
   * will be equal to <code>{2, 1, 0, 0, 0, 2, 0, 3}</code>.</p>
   */
  public final int[] frequencyByMark;

  /**
   * See {@link #frequencyByMark} for explanations. For the example being provided there
   * {@link #lowestMark}'s value will be -3.
   */
  public final int lowestMark;

  /**
   * <p>Creating marking frequencies.</p>
   *
   * @param markValues A non-empty unsorted list of mark values e.g. <code>{-3, 4 , 2, -2, 4, -3, 2, 4}</code>
   *   likely containing duplicates.
   */
  public MarkFrequencies(final int[] markValues) {

    // Step 1: We need to find the minimum and maximum value

    // Starting from the first mark:
    //
    int maxValue = markValues[0],
        minValue = maxValue;

    for (int i = 1; i < markValues.length; i++) {
      final int currentMark = markValues[i];
      if (maxValue < currentMark) {
        maxValue = currentMark;
      }
      if (currentMark < minValue) {
        minValue = currentMark;
      }
    }
    lowestMark = minValue;

    // Step 2: Create the array of frequencies based on the now known range of marks
    frequencyByMark = new int[maxValue - minValue + 1];

    // Collect the values' frequencies
    for (final int m : markValues) {
      frequencyByMark[m - minValue]++;
    }
  }
}