package de.hdm_stuttgart.mi.sd1.shapetest;

import org.junit.Assert;
import org.junit.Test;

import de.hdm_stuttgart.mi.sd1.shape.model.Circle;
import de.hdm_stuttgart.mi.sd1.shape.model.Rectangle;

@SuppressWarnings({"UnusedDeclaration"})
public class ShapeTest {
  static private final double accuracyLimit = 1.E-15;  // Rounding accuracy limit with respect to arithmetic operations

  @Test
  public void circleTranslation() {

    final double
      radius = 3.0,
      newRadius = 1.5,
      x = 3.,               y =  4.,         // circle's center coordinates (3, 4)
      transX = 3.,          transY = -2.,         // translation vector (3, -2)
      expectX = x + transX, expectY =  y + transY; // translated circle's coordinates (6, 2)

    final Circle c = new Circle(x, y, radius);
    c.move(transX, transY);

    // Check moved circle's new center coordinate
    Assert.assertEquals(expectX, c.getX(), accuracyLimit);
    Assert.assertEquals(expectY, c.getY(), accuracyLimit);

    Assert.assertEquals(radius,  c.getRadius(), accuracyLimit);

    // Changing radius
    c.setRadius(newRadius);

    // New values present?
    Assert.assertEquals(newRadius,  c.getRadius(), accuracyLimit);

    // center coordinates remain unchanged?
    Assert.assertEquals(expectX, c.getX(), accuracyLimit);
    Assert.assertEquals(expectY, c.getY(), accuracyLimit);
  }

  @Test
  public void rectangleTranslation() {

    final double
      width = 2.,           height = 1.,
      newWidth = 3.,        newHeight = 0.5,
      x = 2.,               y =  6.5,       // rectangle's center coordinates (2, 6.5)
      transX = 4.,          transY =  1.,        // translation vector (3, -2)
      expectX = x + transX, expectY = y + transY; // translated circle's coordinates (6, 2)

    final Rectangle r = new Rectangle(x, y, width, height);

    Assert.assertEquals(width, r.getWidth(), accuracyLimit);
    Assert.assertEquals(height, r.getHeight(), accuracyLimit);

    r.move(transX, transY);

    // Check moved rectangle's new center coordinate
    Assert.assertEquals(expectX, r.getX(), accuracyLimit);
    Assert.assertEquals(expectY, r.getY(), accuracyLimit);

    // Changing width and height
    r.setWidth(newWidth);
    r.setHeight(newHeight);

    // New values present?
    Assert.assertEquals(newWidth, r.getWidth(), accuracyLimit);
    Assert.assertEquals(newHeight, r.getHeight(), accuracyLimit);

    // center coordinates remain unchanged?
    Assert.assertEquals(expectX, r.getX(), accuracyLimit);
    Assert.assertEquals(expectY, r.getY(), accuracyLimit);

    // Change of center coordinates
    r.setX(0.);
    r.setY(0.);

    // center coordinates still unchanged?
    Assert.assertEquals(0., r.getX(), accuracyLimit);
    Assert.assertEquals(0., r.getY(), accuracyLimit);
  }

  @Test
  public void circlePerimeterAndArea() {
    final double
      radius = 1.,
      expectedPerimeter = 2 * Math.PI * radius,
      expectedArea = Math.PI * radius * radius;

    final Circle c = new Circle(3, 4, radius);

    Assert.assertEquals(expectedPerimeter, c.getPerimeter(), accuracyLimit);
    Assert.assertEquals(expectedArea, c.getArea(), accuracyLimit);
  }

  @Test
  public void rectanglePerimeterAndArea() {
    final double
      width = 2.,
      height = 1.,
      expectedPerimeter = 2 * (width + height),
      expectedArea = width * height;

    final Rectangle r = new Rectangle(1,  3, width, height);

    Assert.assertEquals(expectedPerimeter, r.getPerimeter(), accuracyLimit);
    Assert.assertEquals(expectedArea, r.getArea(), accuracyLimit);
  }
  @Test
  public void rectangleScale() {
    final double
      width = 2.,
      height = 1.,
      factor = 3;

    final Rectangle r = new Rectangle(1,  3, width, height);

    final double
      originalPerimeter = r.getPerimeter(),
      originalArea = r.getArea(),
      expectedPerimeter = originalPerimeter * factor, // Perimeter is linear with respect to scale factor
      expectedArea = originalArea * factor * factor;  // Area is quadratic with respect to scale factor

    r.scale(factor);

    Assert.assertEquals(expectedPerimeter, r.getPerimeter(), accuracyLimit);
    Assert.assertEquals(expectedArea, r.getArea(), accuracyLimit);
  }

  @Test
  public void circleScale() {
    final double
      radius = 2.,
      factor = 0.7;

    final Circle c = new Circle(-2,  5, radius);

    final double
      originalPerimeter = c.getPerimeter(),
      originalArea = c.getArea(),
      expectedPerimeter = originalPerimeter * factor, // Perimeter is linear with respect to scale factor
      expectedArea = originalArea * factor * factor;  // Area is quadratic with respect to scale factor

    Assert.assertEquals(expectedPerimeter, c.scale(factor).getPerimeter(), accuracyLimit);
    Assert.assertEquals(expectedArea, c.getArea(), accuracyLimit);
  }
  @Test
  public void circleToString() {
    final Circle c = new Circle(-2, -1, 3.5);
    Assert.assertEquals("Circle (-2.0,-1.0), radius=3.5", c.toString());
  }
  @Test
  public void rectangleToString() {
    final Rectangle r = new Rectangle(3, 1, 1.5, 4.4);
    Assert.assertEquals("Rectangle (3.0,1.0), width=1.5, height=4.4", r.toString());
  }
}
