package de.hdm_stuttgart.mi.sd1.tictactoe;

public class Board {

  static private final int clearTerminalLineCount = 20;                 // Scroll down this many lines simulating terminal clear.
  static private int fieldSize = 3;
  static private int numberOfFields = fieldSize * fieldSize;            // Each Tic-Tac-Toe board has got 3 x 3 = 9 fields
  private short numberOfMoves = 0;                                      // Game will be over after a maximum of 9 moves

  private Player currentActivePlayer;                                   // Player about to start next move

  private final Player[] board = new Player[numberOfFields];            // Allocating a one- dimensional array, yet empty.

  public Board(final Player startingPlayer) {
    currentActivePlayer = startingPlayer; 
    for (int i = 0; i < numberOfFields; i++) {                          // Initialize board with null values.
      board[i] = null;
    }
  }

  public Board(final Player activePlayer, Player[] initial) {           // For unit testing purpose.
    currentActivePlayer = activePlayer;
    for (int i = 0; i < numberOfFields; i++) {                          // Initialize board with null values.
      board[i] = initial[i];
      if (null != board[i]) {
        numberOfMoves++;
      }
    }
  }

  public void nextPerfectMove() {
    final int[] freeFields = getFreeFields();
    
    if (0 == freeFields.length) {
      return;
    }
    
    int drawOrLooseIndex = freeFields[0];                            // Choose a candidate. If not confirmed, we're bound to loose anyway
    
    for (int i = 0; i < freeFields.length; i++) {
      
      final int freeFieldIndex = freeFields[i];
      
      board[freeFieldIndex] = currentActivePlayer;                    // Set empty field to player.
      final int score = getScore(
          board, currentActivePlayer.getOtherPlayer(),
          Helper.excludeIndex(freeFields, i));
      
      if (currentActivePlayer.score == score) {
        currentActivePlayer = currentActivePlayer.getOtherPlayer();
        numberOfMoves++;
        return;                                                       // Leave current field occupied by active player.
      } else {
        if (0 == score) {
          drawOrLooseIndex = freeFieldIndex;                          // Better off than loosing.        
        }
      }
      board[freeFieldIndex] = null;                                   // Reset field to empty state, continue searching.
    }
    board[drawOrLooseIndex] = currentActivePlayer;
    numberOfMoves++;
    currentActivePlayer = currentActivePlayer.getOtherPlayer();
  }
  
  
  private int[] getFreeFields() {
    final int[] freeFields = new int[numberOfFields - numberOfMoves];

    int index = 0;
    for (int i = 0; i < numberOfFields; i++) {
      if (null == board[i]) {
        freeFields[index++] = i;
      }
    }
    return freeFields;
  }

  public int getScore() {                                               // For unit testing purpose.
    final int[] freeFields = getFreeFields();
    return getScore(board, currentActivePlayer, freeFields);
  }

  private int getScore(final Player[] evalBoard, final Player activePlayer, final int[] freeFields) {

    {
      final Player winner = getWinner(evalBoard);

      if (null != winner) { 
        return winner.score;                          // "Winning" end state.
      } else if (0 == freeFields.length) {
        return 0;                                     // "Draw" end state: Board completely populated.
      }
    }

    boolean drawChildExists = false;
    for(int i = 0; i < freeFields.length; i++) {
      final int freeFieldIndex = freeFields[i];

      evalBoard[freeFieldIndex] = activePlayer;       // Set empty field to player.
      final int score = getScore(
          evalBoard, activePlayer.getOtherPlayer(),
          Helper.excludeIndex(freeFields, i));
      evalBoard[freeFieldIndex] = null;               // Reset field to empty state.

      if (activePlayer.score == score) {              // Maximum or minimum (with respect to active player) has been reached.
        return activePlayer.score;
      } else if (0 == score) {
        drawChildExists = true;                       // Continue searching for better (maximal or minimal) score.
      }
    }

    if (drawChildExists) {
      return 0;                                       // There is at least one "draw" child to choose from.
    } else {
      return activePlayer.getOtherPlayer().score;     // All board children score to the active player's enemy
    }
  }

  Player evaluateWinner(){
    return getWinner(board);
  }

  /**
   * Test whether one of our two players has won the game.
   * 
   * @return The winner instance if a winner exists, null otherwise
   */
  static private Player getWinner(Player[] evalBoard) {

    if (null != evalBoard[4] && (                                            // Check both diagonals
        (evalBoard[4] == evalBoard[0] && evalBoard[4] == evalBoard[8]) ||                //
        (evalBoard[4] == evalBoard[6] && evalBoard[4] == evalBoard[2]))) {               //   x x
      return evalBoard[4];                                                   //    x
    }                                                                    //   x x

    for (int i = 0; i < fieldSize; i++) {                                // Check all three columns
      if (null != evalBoard[i] &&                                            //  +-=                                      
          evalBoard[i] == evalBoard[i + fieldSize] &&
          evalBoard[i] == evalBoard[i + 2* fieldSize]) {                         //  +-=
        return evalBoard[i];                                                 //  +-=
      }
    }

    for (int i = 0; i < 2 * fieldSize + 1; i += fieldSize) {             // Check all three rows
      if (null != evalBoard[i] &&                                            // +++
          evalBoard[i] == evalBoard[i + 1] && evalBoard[i] == evalBoard[i + 2]) {        // ---
        return evalBoard[i];                                                 // ===
      }
    }
    return null;
  }

  /**
   * Occupy the next field by player.
   * 
   * Fields are being numbered by:
   * 
   *  1|2|3
   *  -+-+-
   *  4|5|6
   *  -+-+-
   *  7|8|9
   * 
   * @param field The field in question
   *  
   * @return An error message indication either an occupied field or an index violation.
   *          null if everything is o.K.
   */
  public String nextMove(final int field) {                          

    if (field < 1 || numberOfFields < field) {
      return "Field index out of range [0 ..." + 
          numberOfFields + "]";
    } else {
      final int fieldIndex = field - 1;                               // User to (computer) nerd point of view
      final Player current = board[fieldIndex];
      if (current == null) {
        board[fieldIndex] = currentActivePlayer;
        currentActivePlayer = currentActivePlayer.getOtherPlayer();
        numberOfMoves++;
        return null;
      } else {
        return "Field already occupied by " + current.nickname;
      }
    }
  }

  /**
   * After nine moves all fields will have been occupied.
   * @return true if all possible moves have been completed, false otherwise.
   */
  boolean allMovesFinished() {
    return numberOfFields <= numberOfMoves;
  }

  public void print(boolean simulateClear, boolean printNumberingHints) {
    print(board, simulateClear, printNumberingHints);
  }

  /**
   * Publish the current state to standard output using
   * ASCII graphics.
   */
  static private void print(final Player[] boardState, boolean simulateClear, boolean printNumberingHints) {
    for (int clear = 0; simulateClear && clear < clearTerminalLineCount; clear++) {
      System.out.println("\n");
    }

    System.out.println("\n\nPlayer " + Player.YOU.nickname + "("
        + Player.YOU.representation + ")" + "\nvs. " + Player.COM.nickname
        + "(" + Player.COM.representation + ")"  + "          Free fields\n");

    final String tableSeparator = "                  ";
    for (int x = 0; x < fieldSize; x++) {
      
      // Print current values
      for (int y = 0; y < fieldSize; y++) {
        final int fieldIndex = y + fieldSize * x;
        System.out.print(null == boardState[fieldIndex] ? " " : boardState[fieldIndex].representation);
        if (y < fieldSize - 1) {
          System.out.print('|'  );
        } 
      }
      if (printNumberingHints) {
      System.out.print(tableSeparator);
      
      // print free field numbering hints.
      for (int y = 0; y < fieldSize; y++) {
        final int fieldIndex = y + fieldSize * x;
        System.out.print(null == boardState[fieldIndex] ? fieldIndex + 1 : // Nerd to user perspective.
            " ");
        if (y < fieldSize - 1) {
          System.out.print('|');
        }
      }
      }

      if (x < fieldSize - 1) {
        System.out.print("\n-+-+-" + (printNumberingHints ? tableSeparator + "-+-+-" : "") );
        System.out.println();
      }
    }
  }
  public Player[] getBoard() {
    return board.clone();
  }  
}
