package de.hdm_stuttgart.mi.sd1.interest;

/**
 * Dealing with account balances and interest calculations.
 * 
 */
public class Account {

  private static double         // All accounts share a common interest rate. Thus
  interestRate = 1.5;           // this parameter is being declared as "static".

  private double balance;       // Balances apply per-account thus in contrast
                                // "static" must not appear here.

  /**
   * Create a new account having a balance of 0.
   */
  public Account() {            // Default Constructor. This one has no parameters and
    setBalance(0);              // thus creates an initially empty account. 
  }

  /**
   * Create a new account having of given initial balance.
   * 
   * @param balance 
   *           The account's initial balance.
   */
  public Account(double balance) {  // Non-default constructor creating an account
    setBalance(balance);            // with (possibly) non-zero balance.
  }

  /**
   * @return The account's current balance.
   */
  public double getBalance() {
    return balance;
  }
  /**
   * Setting a (possibly) new balance.
   * 
   * @param balance
   *      The desired new balance value. If greater than 10000
   *      a warning message is being issued and the accounts current value will be retained.
   */
  public void setBalance(double balance) {
    if (balance <= 10000) {
      this.balance = balance; // "this" required to resolve name shadowing conflict of formal
                              // parameter name "double stand" with respect to Account.stand .
    } else {
      System.out.println("Balance" + balance + " exceeds " + 10000);
    }
  }
  /**
   * Setting the interest rate common to all accounts.
   * 
   * @param z
   *            the desired (global) interest rate.
   */
  public static void setInterestRate(double z) {  // Scope of variable "z" limited is just the next block {...},
    interestRate = z;                             // in contrast interestRate has class scope.
  }
  /**
   * @return the current globally used interest rate
   */
  public static double getInterestRate() {
    return interestRate;
  }
  /**
   * <p>Adding the annual interest to the current balance according to:</p>
   *
   * \[ balance_{1} = balance (1 + {interestRate\over 100}) \]
   */
  public void applyInterest() {
    balance = balance * (1 + interestRate / 100);
  }

  /**
   * <p>Adding the interest of several years to the current balance according to:</p>
   *
   * \[ balance_{years} = balance\ {\left(1 + {interestRate\over 100}\right)}^{years} \]
   *
   * @param years the given time period.
   */
  public void applyInterest(int years) {
    balance = balance * Math.pow((1 + interestRate / 100), years) ; // interest factor raised to the power of years, see javadoc.
  }

  /**
   * Calculate the expected annual interest without changing the account's balance.
   * 
   * @return Annual interest to be expected
   */
  public double getYearlyInterest() {
    return balance * interestRate / 100;
  }
}