package de.hdm_stuttgart.mi.sda1.hashprovider;

import javax.crypto.SecretKey;
import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;

import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;

import org.apache.commons.codec.binary.Base64;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

/**
 * Credits to http://stackoverflow.com/users/90998/martin-konicek, answer #12 in
 * http://stackoverflow.com/questions/2860943/suggestions-for-library-to-hash-passwords-in-java
 * 
 * Subsequent code modified from original.
 * 
 */
public class HashProvider {

	static final Logger log;
	static final SecretKeyFactory secretKeyFactory; 
	static final SecureRandom secureRandom;

	final byte[] salt;
	
	static {
		final Logger logTmp = LogManager.getLogger(HashProvider.class);
		log = logTmp;

		SecureRandom secureRandomTmp = null;
		final String securRandomAlgorithm = "SHA1PRNG";
		try {
			secureRandomTmp = SecureRandom.getInstance(securRandomAlgorithm);
		} catch (final NoSuchAlgorithmException e) {
			logTmp.fatal("No such algorithm '" + securRandomAlgorithm + "'");
		} 
		secureRandom = secureRandomTmp;

		SecretKeyFactory secretKeyFactoryTmp = null;
		final String secretKeyAlgorithm = "PBKDF2WithHmacSHA512";//PBKDF2WithHmacSHA1
		try {
			secretKeyFactoryTmp = SecretKeyFactory.getInstance(secretKeyAlgorithm);
		} catch (final NoSuchAlgorithmException e) {
			logTmp.fatal("No such algorithm '" + secretKeyAlgorithm + "'");
		} 
		secretKeyFactory = secretKeyFactoryTmp;
	}

	public HashProvider(int saltLength) {
		salt = secureRandom.generateSeed(saltLength);  // This may take some time depending on /dev/urandom's underlying entropy pool
	}
	
	/**
	 * Separator between password hash and seed. Consider:
	 * 
	 * GWTllfNq1vUHaY2iGm+QI1i21fUOEaU7shVvHzNgi+g=$v3juEYYE4fNAqAVnkQLROr8E2hReZHxE2ijpxcey5Yc=
	 * 
	 * With "$" being the separator token "GWTllfNq1vUHaY2iGm+QI1i21fUOEaU7shVvHzNgi+g="
	 * will be the password's hash and "v3juEYYE4fNAqAVnkQLROr8E2hReZHxE2ijpxcey5Yc="
	 * its corresponding seed both being base64 encoded.
	 */
	static public final char separatorToken = '$';

	// The higher the number of iterations the more 
	// expensive computing the hash is for us
	// and also for a brute force attack.
	private static final int iterations = 10*1024;
	private static final int desiredKeyLen = 256;

	/**
	 * See {@link #getSaltedHash(char[])}.
	 * @param password .
	 * @return .
	 */
	public String getSaltedHash(String password) {
		return getSaltedHash(password.toCharArray());
	}

	/**
	 * Computes a salted PBKDF2 hash of given plain text password
        suitable for storing in a database

	 * @param password The plain text password
	 * @return the generated hash separated from salt by "$"
	 */
	public String getSaltedHash(char [] password) {
		// store the salt with the password

		return Base64.encodeBase64String(salt) + separatorToken +
				hash(password, salt);
	}

	/**
	 * Checks whether given plain text password corresponds 
        to a stored salted hash of the password.
	 * @param password The plain text password
	 * @param stored the salt/hash to be verified against
	 * @return true if input matches, false otherwise
	 */
	public static boolean check(final char[] password, final String stored) {

		if (null == password || null == stored) {
			return false;
		} else { // Find hash / seed separator token
			final int separatorPosition = stored.indexOf(separatorToken);

			if (separatorPosition < 0) {
				log.info("No salt detected!");
				return false;
			} else {
				final String hashOfInput = hash(password,Base64.decodeBase64(
						stored.substring(0, separatorPosition)));
				return null != hashOfInput && hashOfInput.equals(
						stored.substring(1 + separatorPosition));
			}
		}
	}

	// using PBKDF2 from Sun, an alternative is https://github.com/wg/scrypt
	// cf. http://www.unlimitednovelty.com/2012/03/dont-use-bcrypt.html
	private static String hash(char [] password, byte[] salt) {
		try {

			final PBEKeySpec pbeKeySpec = new PBEKeySpec(password, salt, iterations, desiredKeyLen); 
			final SecretKey key = secretKeyFactory.generateSecret(pbeKeySpec);
			return Base64.encodeBase64String(key.getEncoded());
		}  catch (final InvalidKeySpecException e) {
			log.fatal("Unable to create hash: " + e);
		}
		return null;
	}
}