package dom;

import java.io.IOException;
import java.io.PrintStream;

import org.jdom2.Comment;
import org.jdom2.DocType;
import org.jdom2.Document;
import org.jdom2.Element;
import org.jdom2.Text;
import org.jdom2.output.Format;
import org.jdom2.output.XMLOutputter;

/**
 * Represent a XHTML DOM for output generation.
 * @author goik
 */
public class HtmlTree {

   private final Element tableBody;

   /**
    * Create a HTML skeleton (&lt;html&gt;&lt;head&gt;...&lt;body&gt; ...&lt;/body&gt;&lt;/head&gt;
    * to be filled in later by calling {@link #appendItem(Element)}.
    *
    * @param titleText 
    *   The document's title, e.g. "Available articles"
    *   
    * @param tableHeaderFields
    *   The articles will be displayed as HTML table with these header fields
    *   e.g. {"Article Description", "Order Number" }
    */
   public HtmlTree(final String titleText,
         final String[] tableHeaderFields) {

      final DocType doctype =  new DocType("html");

      final Element htmlRoot = new Element("html");
      
      final Document htmlOutput = new Document (htmlRoot, doctype);
      htmlOutput.addContent(0, new Comment(" Static content section"));
      
      // We create a HTML skeleton including a yet empty table
      final Element 
            head = new Element("head"),
            body = new Element("body"),
            table = new Element("table");

      htmlRoot.addContent(head).addContent(body);

      head.addContent(new Element("title").addContent(new Text(titleText)));
      
      body.addContent(new Element("h1").addContent(new Text(titleText))).addContent(table);

      tableBody = new Element("tbody");
      table.addContent(tableBody);
      
      final Element tr = new Element("tr");
      tableBody.addContent(tr);
      
      for (final String headerField:  tableHeaderFields) {
         tr.addContent(new Element("th").addContent(new Text(headerField)));
      }
      
      tableBody.addContent(new Comment(" End of static, beginning of dynamic section"));
   }
   
   /**
    * Inserting an &lt;item&gt; as a new table row with two &lt;td&gt; elements.
    * @param item XML representing article.
    * 
    */
   public void appendItem(final Element item) {
      final Element tr = new Element("tr");
      tableBody.addContent(tr);
      tr.addContent(new Element("td").addContent(new Text(item.getText())));
      tr.addContent(new Element("td").addContent(new Text(item.getAttributeValue("orderNo"))));
   }
   /**
    * Serialize the HTML DOM to a stream
    * @param out Output destination.
    */
   public void serialize(final PrintStream out){

      // Set formatting for the XML output
      final Format outFormat = Format.getPrettyFormat();

      // Serialize to console
      final XMLOutputter printer = new XMLOutputter(outFormat);
      try {
         printer.output(tableBody.getDocument(), System.out);
      } catch (IOException e) {
         e.printStackTrace();
         System.exit(1);
      }
   }
   /**
    * @return the table's &lt;tbody&gt; element
    */
   public Element getTable() {
      return tableBody;
   }
}