package de.hdm_stuttgart.mi.javastreams;

import com.google.common.collect.ImmutableList;

import java.util.List;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.function.Predicate;

/**
 * Loosely adapted and modified from
 * https://docs.oracle.com/javase/tutorial/java/javaOO/examples/Person.java
 * and
 * https://docs.oracle.com/javase/tutorial/java/javaOO/examples/RosterTest.java
 */
@SuppressWarnings({"UnusedDeclaration"})
public class RosterTest {

  static private final List<Student> students = ImmutableList.of(
      new Student("Fred", 2, Student.Sex.MALE, "fred@example.com")
      ,new Student("Jane", 1, Student.Sex.FEMALE, "jane@kiv.de")
      ,new Student("George", 4, Student.Sex.MALE, "george@math.edu")
      ,new Student("Bob", 2, Student.Sex.MALE, "bob@uk.edu")
      ,new Student("Kim", 2, Student.Sex.FEMALE, "wilde@serious.de")
  );


  interface CheckStudent {
    boolean test(Student p);
  }

  /**
   * Approach 1: Create Methods that searching for Persons matching one characteristic
   *
   * @param mark .
   */
  private static void printPersonsOlderThan(final int mark) {
    for (final Student p : students) {
      if (p.getMark()>= mark) {
        System.out.println(p);
      }
    }
  }

  /**
   * Approach 2: Create more generalized search methods
   *
   * @param low Lower mark limit
   * @param high upper mark limit
   */
  private static void printPersonsWithinMarkRange(final int low, final int high) {
    for (final Student p : students) {
      if (low <= p.getMark() && p.getMark() <= high) {
        System.out.println(p);
      }
    }
  }

  /**
   * Approach 3: Specify search criteria in a local class
   * Approach 4: Specify search criteria by an anonymous class
   * Approach 5: Specify search criteria by lambda expression
   *
   * @param tester Student instance selector.
   */
  private static void printPersons(final CheckStudent tester) {
    for (final Student p : students) {
      if (tester.test(p)) {
        System.out.println(p);
      }
    }
  }

  /**
   * Approach 6: Use standard functional interfaces employing lambda expressions.
   *
   * @param tester Student instance selector.
   */
  private static void printPersonsWithPredicate(final Predicate<Student> tester) {
    for (final Student p : students) {
      if (tester.test(p)) {
        System.out.println(p);
      }
    }
  }

  /**
   * Approach 7: Use lambda expressions throughout your application
   *
   * @param tester .
   * @param block .
   */
  private static void processPersons(final Predicate<Student> tester,
                                    final Consumer<Student> block) {
    for (final Student p : students) {
      if (tester.test(p)) {
        block.accept(p);
      }
    }
  }

  /**
   * Approach 7, second example
   *
   * @param tester .
   * @param mapper .
   * @param block .
   */
  private static void processPersonsByFunction(final Predicate<Student> tester,
                                              final Function<Student, String> mapper, final Consumer<String> block) {
    for (final Student p : students) {
      if (tester.test(p)) {
        String data = mapper.apply(p);
        block.accept(data);
      }
    }
  }

  /**
   * Approach 8: Use Generics more extensively
   *
   * @param source .
   * @param tester .
   * @param mapper .
   * @param block .
   */
  private static <X, Y> void processElements( final Iterable<X> source, final Predicate<X> tester,
                                             final Function<X, Y> mapper, final Consumer<Y> block) {
    for (final X p : source) {
      if (tester.test(p)) {
        Y data = mapper.apply(p);
        block.accept(data);
      }
    }
  }

  /**
   * @param args unused
   */
  public static void main(final String[] args) {

    System.out.println("All student records:");
    for (final Student p : students) {
      System.out.println(p);
    }

    // Approach 1: Create methods searching for students matching one characteristic
    System.out.println("\nApproach 1: Persons older than 20:");
    printPersonsOlderThan(20);

    // Approach 2: Create more generalized search methods
    System.out.println("\nApproach 2: Persons between 14 and 30 of age:");
    printPersonsWithinMarkRange(14, 30);

    // Approach 3: Specify search criteria by a local class
    System.out.println("\nApproach 3:  Persons eligible for selective service:");
    class CheckStudentEligibleForSelectiveService implements CheckStudent {
      @Override
      public boolean test(Student p) {
        return p.getSex() == Student.Sex.MALE
            && p.getMark() >= 2
            && p.getMark() <= 3;
      }
    }
    printPersons(new CheckStudentEligibleForSelectiveService());

    // Approach 4: Specify search criteria code in an anonymous class
    System.out.println("\nApproach 4: Persons eligible for selective service " +
        "(anonymous class):");
    printPersons(
        new CheckStudent() {
          @Override
          public boolean test(final Student p) {
            return p.getSex() == Student.Sex.MALE
                && p.getMark() >= 2
                && p.getMark() <= 3;
          }
        }
    );

    // Approach 5: Specify search criteria by lambda expression
    System.out.println("\nApproach 5: Persons eligible for selective service " +
        "(lambda expression):");

    printPersons(
        (Student p) -> p.getSex() == Student.Sex.MALE
            && p.getMark() >= 2
            && p.getMark() <= 3
    );

    // Approach 6: Use Standard functional interfaces employing lambda expressions
    System.out.println("\nApproach 6: Persons eligible for selective service " +
        "(with Predicate parameter):");

    printPersonsWithPredicate(
        p -> p.getSex() == Student.Sex.MALE
            && p.getMark() >= 18
            && p.getMark() <= 25
    );

    // Approach 7: Using lamba expressions throughout
    System.out.println("\nApproach 7: Persons eligible for selective service " +
        "(with Predicate and Consumer parameters):");

    processPersons(
        p -> p.getSex() == Student.Sex.MALE
            && p.getMark() >= 2
            && p.getMark() <= 3,
        p -> System.out.println(p)
    );

    // Approach 7, second example
    System.out.println("\nApproach 7, second example: Persons eligible for selective service " +
        "(with Predicate, Function, and Consumer parameters):");

    processPersonsByFunction(
        p -> p.getSex() == Student.Sex.MALE
            && p.getMark() >= 2
            && p.getMark() <= 3,
        p -> p.getEmailAddress(),
        email -> System.out.println(email)
    );

    // Approach 8: Use generics more extensively
    System.out.println("\nApproach 8: Persons eligible for selective service " +
        "(generic version):");

    processElements(
        students,
        p -> p.getSex() == Student.Sex.MALE
            && p.getMark() >= 2
            && p.getMark() <= 3,
        p -> p.getEmailAddress(),
        email -> System.out.println(email)
    );

    // Approach 9: Using bulk data operations accepting lambda expressions
    // as Parameters

    System.out.println("\nApproach 9: Persons eligible for selective service " +
        "(with bulk data operations):");

    students
        .stream()
        .filter(
            p -> p.getSex() == Student.Sex.MALE
                && p.getMark() >= 2
                && p.getMark() <= 3)
        .map(p -> p.getEmailAddress())
        .forEach(email -> System.out.println(email));
  }
}
