package de.hdm_stuttgart.mi.exam.unitmarking.xml;

import org.junit.Assert;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.stream.StreamSource;
import javax.xml.validation.SchemaFactory;
import javax.xml.validation.Validator;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.Map;

public class XmlTestSupport {

    /**
     * <p>The XML Schema namespace as being defined in
     * https://www.w3.org/TR/xmlschema-1/#Instance_Document_Constructions.</p>
     */
    static public final String
            xmlns_xsi = "http://www.w3.org/2001/XMLSchema-instance",
            /**
             * <p>The schema reference attribute name, see {@link #xmlns_xsi} for explanation.</p>
             */
            noNamespaceSchemaLocationAttributeName = "noNamespaceSchemaLocation";

    // XMLConstants only provides W3C_XML_SCHEMA_NS_URI rather than XSD-1.1;
    static private final SchemaFactory schemaFactory = SchemaFactory.newInstance(
        org.apache.xerces.impl.Constants.W3C_XML_SCHEMA11_NS_URI);
    private final DocumentBuilder nonValidatingBuilder;

    /**
     * The directory containing schema unit tests.
     */
    public final String testDirectoryPathname;


    /**
     * Unit test support for schema unit tests.
     *
     * @param testDirectoryPathname see {@link #testDirectoryPathname}
     */
    public XmlTestSupport(final String testDirectoryPathname) {
        this.testDirectoryPathname = testDirectoryPathname;
        final DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(false);
        factory.setXIncludeAware(true);
        DocumentBuilder nonValidatingBuilderTmp = null;
        try {
            nonValidatingBuilderTmp = factory.newDocumentBuilder();
            nonValidatingBuilderTmp.setErrorHandler(new DefaultHandler());
        } catch (ParserConfigurationException e) {
            Assert.fail("Unable to create non-validating parser:" + e);
        } finally {
            nonValidatingBuilder = nonValidatingBuilderTmp;
        }
    }

    /**
     * Assert whether or not the given instance is valid.
     *
     * @param instanceFilename    The XML instance containing an XML schema reference to be handled.
     * @param expectValidInstance A value of true of false will cause the assertion to fail in case the instance is
     *                            invalid or valid respectively.
     */
    private void assertExpectedValidationStatus(final String instanceFilename, final boolean expectValidInstance) {

        final Path instancePath = Paths.get(testDirectoryPathname, instanceFilename).normalize();

        final Attr schemaPathNameAttr = parseWellFormed(  // Schema reference found?
                instancePath).getAttributeNodeNS(xmlns_xsi, noNamespaceSchemaLocationAttributeName);

        Assert.assertNotNull(  // Oops, no schema!
                "No schema provided by attribute " + noNamespaceSchemaLocationAttributeName
                        + " from namespace\n" + xmlns_xsi + " in file " + getClickableFileName(instanceFilename, 1),
                schemaPathNameAttr);

        final Path schemaPath = Paths.get(
                instancePath.getParent().toString(),        // Strip filename from path,
                schemaPathNameAttr.getValue()).normalize(); // add relative schema path

        parseWellFormed(schemaPath);  // Schema is well-formed?
        final Path xmlSchemaPath = Paths.get("src/main/resources/XMLSchema.xsd");
        validate(schemaPath, xmlSchemaPath, true);// Schema valid with respect to XML schema standard ?
        validate(instancePath, schemaPath, expectValidInstance);  // Instance valid or invalid  with respect to given XML schema ?
    }

    /**
     * <p>Assert a given instance to be valid. The following steps will be evaluated:</p>
     *
     * <ol>
     *     <li>
     *         <p>Check instance for well-formedness</p>
     *         <p>Possible failure: Instnce may not be well-formed.</p>
     *     </li>
     *     <li>
     *         <p>Get the associated schema using the instance's {@link #noNamespaceSchemaLocationAttributeName}
     *      *     attribute value.</p>
     *         <p>Possible failures:</p>
     *         <ul>
     *             <li>Missing attribute or wrong namespace</li>
     *             <li>Missing or wrong attribute value not being resolvable to an XML Schema.</li>
     *         </ul>
     *     </li>
     *     <li>
     *         <p>Check the associated schema to be well-formed.</p>
     *         <p>Possible failure: The schema may not be well-formed.</p>
     *     </li>
     *     <li>
     *         <p>Check the associated schema itself to be valid with respect to
     *      *     <a href="https://www.w3.org/TR/xmlschema-1/#normative-schemaSchema">A Schema for Schemas</a>.</p>
     *         <p>Possible failure: The schema may not be well-formed.</p>
     *     </li>
     *     <li id="valid2schema">
     *         <p>Check the instance for validity with respect to the associated schema.</p>
     *         <p>Possible failure: The instance may not be valid.</p>
     *     </li>
     * </ol>
     *
     * <p>Any negative outcome will result in a test failure.</p>
     *
     * @param instanceFilename The instance to be examined.
     */
    public void assertValid(final String instanceFilename) {
        assertExpectedValidationStatus(instanceFilename, true);
    }

    /**
     * <p>Similar to {@link #assertValid(String)} except for the <a href="#valid2schema">very last step</a> being
     * expected to fail.</p>
     *
     * @param instanceFilename The instance to be examined.
     */
    public void assertInvalid(final String instanceFilename) {
        assertExpectedValidationStatus(instanceFilename, false);
    }

    static void validate(final Path instancePath, final Path schemaPath, final boolean expectValidInstance) {

        final String instanceFilename = instancePath.getFileName().toString();
        try {

            // Weird: Caching validators by schema for re-use fails
            // on xs:ID attributes on repeated parsing!
            // So always re-create!
            final Validator validator = schemaFactory.newSchema(schemaPath.toFile()).newValidator();
            validator.setFeature("http://apache.org/xml/features/validation/schema", true);
            validator.setFeature("http://apache.org/xml/features/validation/assert-comments-and-pi-checking", true);

            validator.validate(                               // Re-do parsing: Starting from DOMSource(root)
                    new StreamSource(instancePath.toFile())); // conceals line numbers, sigh!
            if (!expectValidInstance) {
                Assert.fail("Instance " + getClickableFileName(instanceFilename, 1) + " is supposed to be invalid");
            }
        } catch (final SAXParseException e) {
            if (expectValidInstance) {
                Assert.fail("Instance " + getClickableFileName(instanceFilename, e.getLineNumber()) +
                        " is invalid:\n" + format(e) + "\n");
            }
        } catch (final SAXException e) {
            if (expectValidInstance) {
                Assert.fail("Instance " + instanceFilename + " is NOT valid reason:" + e + "\n");
            }
        } catch (final IOException e) {
            Assert.fail("Unable to read " + instanceFilename + ": " + e + "\n");
        }
    }

    private Element parseWellFormed(final Path instancePath) {
        try {
            return nonValidatingBuilder.parse(
                    instancePath.toFile()).getDocumentElement();
        } catch (final SAXParseException e) {
            Assert.fail("Unable to parse " + instancePath.toString() +
                    ":\n " + getClickableFileName(instancePath.getFileName().toString(), e.getLineNumber()) + ")\n" +
                    format(e) + "\nDocument possibly not well-formed\n");
        } catch (final SAXException e) {
            Assert.fail("Unable to parse '" + instancePath + "':\n" + e);
        } catch (final IOException e) {
            Assert.fail("Unable to read '" + instancePath + "':\n" + e);
        }
        return null;  // Runtime unreachable due to previously either returning or assert based test failure.
    }

    static private String format(final SAXParseException e) {
        return "line=" + e.getLineNumber() + ", column=" + e.getColumnNumber() + ":\n" +
                e.getMessage();
    }

    // See
    // https://stackoverflow.com/questions/7930844/is-it-possible-to-have-clickable-class-names-in-console-output-in-intellij
    static private String getClickableFileName(final String fileName, final int line) {
        return ".(" + fileName + ":" + line + ")";
    }
}