package de.hdm_stuttgart.mi.unitmarking.xsd;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Vector;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.Document;
import org.jdom2.JDOMException;
import org.jdom2.ProcessingInstruction;
import org.jdom2.filter.Filters;
import org.jdom2.input.SAXBuilder;
import org.jdom2.input.sax.XMLReaders;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

/**
 * Evaluate a given set of XML document instances
 *
 */
public class InstanceSetEvaluation {

	/**
	 * Base directory containing xml test file instances.
	 */
	public final String xmlTestFileDir;

	/**
	 * true if all tests have been successful.
	 */
	public final boolean allTestsSucceeded;

	private static Logger log = LogManager.getLogger(InstanceSetEvaluation.class);

	final static List<InstanceTest> tests = new Vector<>();
	final static StringBuffer warnings = new StringBuffer();

	final StringBuffer messages = new StringBuffer(), errorMessages = new StringBuffer();

	/**
	 * Read the list of all processing instructions (PIs) of a given XML file.
	 *
	 * @param instanceFilename The xml source to be searched for PIs.
	 * @return The list of all PIs.
	 *
	 */
	static public List<ProcessingInstruction> getProcessingInstructions(final File instanceFilename) {
		final SAXBuilder metainfoParser = new SAXBuilder(XMLReaders.NONVALIDATING);
		try {
			final Document doc = metainfoParser.build(instanceFilename);

			final XPathExpression<ProcessingInstruction> searchHeader =
					XPathFactory.instance().compile(
							"/processing-instruction('xmlTest')",
							Filters.processinginstruction());
			return searchHeader.evaluate(doc);
		} catch (final JDOMException e) {
			log.error("Document '" + instanceFilename.getPath() +
					"' is invalid:", e);
			warnings.append("Unable to parse document " + instanceFilename.getPath() + ": " + e + "\n");
		} catch (final IOException e) {
			warnings.append("Unable to read document " + instanceFilename.getPath() + ": " + e + "\n");
		}
		return new ArrayList<>();
	}

	/**
	 * @return Individual failed test(s) error message(s).
	 */
	public String getErrorMessages() {
		return errorMessages.toString();
	}

	/**
	 * @return Individual test's and aggregated results.
	 */
	public String getMessages() {
		return "\n" + warnings.toString() +  "\n" + messages.toString();
	}

	/**
	 * 
	 * @param xmlTestFileDir Directory containing XML instances to be processed.
	 * @param xsdSchemaFilename Schema defining validation rules.
	 */
	public InstanceSetEvaluation(final String xmlTestFileDir, final String xsdSchemaFilename) {
		this.xmlTestFileDir = xmlTestFileDir;

		final File rootDirectory = new File(xmlTestFileDir);

		final File[] xmlFiles = rootDirectory.listFiles(
            path -> path.getPath().endsWith(".xml"));
		
		Arrays.sort(xmlFiles);
		
		for (final File f: xmlFiles) {
			readTestHeader(f, xsdSchemaFilename);
		}

		tests.forEach(t -> messages.append(t + "\n"));      
		tests.stream().filter(t -> !t.testSucceeded).forEach(t -> errorMessages.append(t + "\n"));      

		allTestsSucceeded = tests.
				stream().
				map(t -> t.testSucceeded).
				reduce((a, b) -> a && b).
				get(); 

		final int maxPoints, reachedPoints;
		{
			int tmpMaxPoints = 0, tmpReachedPoints = 0;

			for (final InstanceTest t: tests) {
				tmpMaxPoints += t.reachablePoints;
				if (t.testSucceeded) {
					tmpReachedPoints += t.reachablePoints;
				}
			}
			maxPoints = tmpMaxPoints;
			reachedPoints = tmpReachedPoints;
		}

		messages.append(reachedPoints + " of " + maxPoints +
				" points have been reached");
	}

	void readTestHeader(final File instanceFilename, final String xsdSchemaFilename) {

		final List<ProcessingInstruction> xmlTestList = getProcessingInstructions(instanceFilename);

		switch (xmlTestList.size()) {
			case 0:
				log.info("No 'xmlTest PI found, possible dependency file");
				break;
			case 1:
				tests.add(new InstanceTest(xmlTestList.get(0), xmlTestFileDir, xsdSchemaFilename, instanceFilename));
				break;
			default:
				warnings.append("Found " + xmlTestList.size() + "' annotations in file '" +  instanceFilename.getPath() + "'\n");
		}
	}
}