package de.hdm_stuttgart.mi;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.jdom2.*;
import org.jdom2.filter.Filters;
import org.jdom2.input.SAXBuilder;
import org.jdom2.output.Format;
import org.jdom2.output.LineSeparator;
import org.jdom2.output.XMLOutputter;
import org.jdom2.xpath.XPathExpression;
import org.jdom2.xpath.XPathFactory;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;
import java.util.regex.Pattern;

class TargetDoc {
  private static Logger log = LogManager.getLogger(TargetDoc.class);


  static private final String mappingFileName = "/ma/goik/urlMappings.xml";

  private final XPathFactory xFactory = XPathFactory.instance();
  private final Namespace xlinkNamespace =
      Namespace.getNamespace("xlink", "http://www.w3.org/1999/xlink");

  private Document root;

  final List<String>
      missingUrlReplacements = new Vector<>(),
      provenUrlSet = new Vector<>();

  private Map<String, String > missingUrlSet = new HashMap<>();

  TargetDoc (final String filename) {

    loadLinks();

    final SAXBuilder builder = new SAXBuilder();
    try {
      root = builder.build(new File(filename));
    } catch (final JDOMException e) {
      Check.exitOnError("Unable to parse »" + filename + "«: " + e, 1);

    } catch (final IOException e) {
      Check.exitOnError("Unable to read »" + filename + "«: " + e, 2);
    }
    descendTree(root);
    final Format f = Format.getRawFormat();
    f.setLineSeparator(LineSeparator.NL);
    final XMLOutputter printer =
        new XMLOutputter(f);

    final String resultFilename = filename + ".result";
    try (final FileOutputStream fos = new FileOutputStream(resultFilename)) {
      printer.output(root, fos);
    }  catch (IOException e) {
      Check.exitOnError("Unable to write result XML file " + resultFilename + ":" + e, 6);
    }
    System.out.println("Processing completed! ------------\n\n");
    missingUrlReplacements.forEach(System.out::println);
  }

  private void loadLinks() {
    try (final FileInputStream fis = new FileInputStream(mappingFileName)) {
      XMLDecoder readLinks = new XMLDecoder(fis);
      missingUrlSet = (Map<String, String>) readLinks.readObject();
    } catch (IOException e) {
      System.out.println("Unable to read from " + mappingFileName);
    }
  }

  private void descendTree(final Document current) {
    final XPathExpression<Element> searchLinks = xFactory.compile(
        "//*[@xlink:href]", Filters.element(), null, xlinkNamespace);
    searchLinks.evaluate(current).forEach(this::handleLink);

    try (final FileOutputStream bos = new FileOutputStream(mappingFileName)) {

      final XMLEncoder xmlEncoder = new XMLEncoder(bos);
      xmlEncoder.writeObject(missingUrlSet);
      xmlEncoder.close();
      bos.flush();

    } catch (IOException e) {
      Check.exitOnError("Unable to write to file " + mappingFileName + ": " + e, 5);
    }
  }

  private void handleLink(final Element element) {
    String uri = element.getAttribute("href", xlinkNamespace).getValue();

    final String replace = missingUrlSet.get(uri);
    boolean replacementCheckWasPositive = false;
    if (null != replace && ! replace.trim().equals("") && doesLinkTargetExist(replace)) {
      element.setAttribute("href", replace, xlinkNamespace);
      final Element parent = element.getParentElement();
      final int hrefElementIndex = parent.indexOf(element);
      uri = replace;
    }
    if (!provenUrlSet.contains(uri)) {
      if (!replacementCheckWasPositive && !doesLinkTargetExist(uri)) {
        missingUrlReplacements.add(uri);
        missingUrlSet.put(uri, "");
      } else {
        System.out.println("Cache hit: " + uri);
        provenUrlSet.add(uri);
      }
    }
  }

  private static boolean doesLinkTargetExist(String urlString)
  {

    if (!(urlString.startsWith("http://") || urlString.startsWith("https://") || urlString.startsWith("ftp://"))) {
      if (urlString.startsWith("/")){
        urlString = "https://freedocs.mi.hdm-stuttgart.de" + urlString;
      } else {
        urlString = "https://freedocs.mi.hdm-stuttgart.de/" + urlString;
      }
    }
    if (urlString.contains("#")) {
      final String[] components = urlString.split("#");
      if (2 != components.length) {
        Check.exitOnError("More than one »#« in " + urlString, 3);
      }
      final String baseUrl = components[0],
          anchor = components[1];

      if (baseUrl.startsWith("https://gitlab.mi.hdm-stuttgart.de/")) { // gitlab does not offer open id/name attribs
        return doesUrlExist(urlString);
      } else {
        return anchorExist(baseUrl, anchor);
      }
    } else {
      return doesUrlExist(urlString);
    }
  }

  private static boolean doesUrlExist(final String urlString) {
    final URL url;
    try {
      url = new URL(urlString);
      // We want to check the current URL
      HttpURLConnection.setFollowRedirects(false);

      HttpURLConnection httpURLConnection = (HttpURLConnection) url.openConnection();

      // We don't need to get data
      httpURLConnection.setRequestMethod("HEAD");

      // Some websites don't like programmatic access so pretend to be a browser
      httpURLConnection.setRequestProperty(
          "User-Agent", "Mozilla/5.0 (Windows; U; Windows NT 6.0; en-US; rv:1.9.1.2) Gecko/20090729 Firefox/3.5.2 (.NET CLR 3.5.30729)");
      int responseCode = httpURLConnection.getResponseCode();

      System.out.println("URL " + urlString + ": " + responseCode);

      return isAcceptedHttpResponseCode(responseCode);
    } catch (Exception e) {
      return false;
    }
  }

  static private boolean isAcceptedHttpResponseCode(final int httpResponseCode) {
    switch (httpResponseCode) {
        case HttpURLConnection.HTTP_OK:
        case HttpURLConnection.HTTP_MOVED_PERM:
        case HttpURLConnection.HTTP_MOVED_TEMP:
          return true;

        default:
          return false;
      }
  }

  private static boolean anchorExist(final String urlString, final String anchor) {
    String htmlLine;
    int noError500Attempts = 0;
    do {
      try {
        final URL url = new URL(urlString);

        final HttpURLConnection con = (HttpURLConnection) url.openConnection();

        con.setRequestProperty(
            "User-Agent", "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/67.0.3396.62 Safari/537.36");
        System.out.println("URL/anchor " + urlString + "#" + anchor + ":" + con.getResponseCode());

        if (isAcceptedHttpResponseCode(con.getResponseCode())) {
          final BufferedReader input = new BufferedReader(new InputStreamReader(con.getInputStream()));

          final Pattern p = getAncorMatcher(anchor);
          while ((htmlLine = input.readLine()) != null) {
            if (p.matcher(htmlLine).find()) {
              return true;
            }
          }
          return false;
        } else { // i.E. docs.oracle.com randomly fails on error 500
          if (urlString.startsWith("https://docs.oracle.com") && noError500Attempts++ < 5) {
            System.out.println("Attempt " + noError500Attempts + ": Try yet another time on " + urlString);
          } else {
            return false;
          }
        }
      } catch (Exception e) {
        System.err.println("Failed reading " + urlString + ":" + e);
      }
    } while (true);
  }

  static Pattern getAncorMatcher(final String anchor) {
    final String escapedAncor = escapeAnchor(anchor);
    return Pattern.compile("(id|ID|name|NAME)[ ]*[=][ ]*['|\"]" + escapedAncor + "['|\"]");
  }

  static String escapeAnchor(final String anchor) {
    return anchor.replaceAll("[(]", "\\\\(").
        replaceAll("[)]", "\\\\)").
        replaceAll("\\[", "\\\\[").
        replaceAll("\\]", "\\\\]").
        replaceAll("%20", " ").
        replaceAll("%3C", "&lt;").
        replaceAll("%3E", "&gt;").
        replaceAll("%5B", "\\\\[").
        replaceAll("%5D", "\\\\]");
  }
}